@@ -2,7 +2,7 @@
                           rkward.cpp  -  description
                              -------------------
     begin                : Tue Oct 29 20:06:08 CET 2002
-    copyright            : (C) 2002-2013 by Thomas Friedrichsmeier 
+    copyright            : (C) 2002-2019 by Thomas Friedrichsmeier
     email                : thomas.friedrichsmeier@kdemail.net
  ***************************************************************************/
 
@@ -15,6 +15,8 @@
  *                                                                         *
  ***************************************************************************/
 
+#include "rkward.h"
+
 // include files for QT
 #include <qtimer.h>
 #include <QDesktopWidget>
@@ -24,6 +26,9 @@
 #include <QMenuBar>
 #include <QStatusBar>
 #include <QInputDialog>
+#include <QVBoxLayout>
+#include <QHBoxLayout>
+#include <QIcon>
 
 // include files for KDE
 #include <kmessagebox.h>
@@ -39,12 +44,10 @@
 #include <krecentfilesaction.h>
 #include <ktoolbar.h>
 #include <kactionmenu.h>
-#include <QIcon>
 #include <KSharedConfig>
 #include <KConfigGroup>
 
 // application specific includes
-#include "rkward.h"
 #include "core/rkmodificationtracker.h"
 #include "plugin/rkcomponentmap.h"
 #include "settings/rksettings.h"
@@ -83,8 +86,6 @@
 #include "windows/rkdebugmessagewindow.h"
 #include "rkconsole.h"
 #include "debug.h"
-#include "version.h"
-
 
 #include "agents/showedittextfileagent.h"	// TODO: see below: needed purely for linking!
 #include "dialogs/rkreadlinedialog.h"	// TODO: see below: needed purely for linking!
@@ -121,6 +122,7 @@ RKWardMainWindow::RKWardMainWindow () : KParts::MainWindow ((QWidget *)0, (Qt::W
 	merge_loads = false;
 	rkward_mainwin = this;
 	RKGlobals::rinter = 0;
+	RKCommonFunctions::getRKWardDataDir(); // call this before any forking, in order to avoid potential race conditions during initialization of data dir
 	RKSettings::settings_tracker = new RKSettingsTracker (this);
 
 	///////////////////////////////////////////////////////////////////
@@ -177,6 +179,8 @@ RKWardMainWindow::~RKWardMainWindow() {
 	delete RKGlobals::tracker ();
 	delete RKGlobals::rInterface ();
 	delete RControlWindow::getControl ();
+	factory ()->removeClient (RKComponentMap::getMap ());
+	delete RKComponentMap::getMap ();
 }
 
 void RKWardMainWindow::closeEvent (QCloseEvent *e) {
@@ -198,8 +202,7 @@ void RKWardMainWindow::doPostInit () {
 	RK_TRACE (APP);
 
 	// Check installation first
-	QFile resource_ver (RKCommonFunctions::getRKWardDataDir () + "resource.ver");
-	if (!(resource_ver.open (QIODevice::ReadOnly) && (resource_ver.read (100).trimmed () == RKWARD_VERSION))) {
+	if (RKCommonFunctions::getRKWardDataDir ().isEmpty ()) {
 		KMessageBox::error (this, i18n ("<p>RKWard either could not find its resource files at all, or only an old version of those files. The most likely cause is that the last installation failed to place the files in the correct place. This can lead to all sorts of problems, from single missing features to complete failure to function.</p><p><b>You should quit RKWard, now, and fix your installation</b>. For help with that, see <a href=\"http://rkward.kde.org/compiling\">http://rkward.kde.org/compiling</a>.</p>"), i18n ("Broken installation"), KMessageBox::Notify | KMessageBox::AllowLink);
 	}
 
@@ -268,6 +271,7 @@ void RKWardMainWindow::doPostInit () {
 	// around on the bus in this case.
 
 	updateCWD ();
+	connect (RKGlobals::rInterface (), &RInterface::backendWorkdirChanged, this, &RKWardMainWindow::updateCWD);
 	setCaption (QString ());	// our version of setCaption takes care of creating a correct caption, so we do not need to provide it here
 }
 
@@ -356,9 +360,28 @@ void RKWardMainWindow::startR () {
 	RK_ASSERT (!RKGlobals::rInterface ());
 
 	// make sure our general purpose files directory exists
-	bool ok = QDir ().mkpath (RKSettingsModuleGeneral::filesPath());
+	QString packages_path = RKSettingsModuleGeneral::filesPath() + "/.rkward_packages";
+	bool ok = QDir ().mkpath (packages_path);
 	RK_ASSERT (ok);
 
+	// Copy RKWard R source packages to general  purpose files directory (if still needed).
+	// This may look redundant at first (since the package still needs to be installed from the
+	// backend. However, if frontend and backend are on different machines (eventually), only  the
+	// filesPath is shared between both.
+	QStringList packages;
+	packages << "rkward.tgz" << "rkwardtests.tgz";
+	for (int i = 0; i < packages.size (); ++i) {
+		QString package = QDir (packages_path).absoluteFilePath (packages[i]);
+		if (RKSettingsModuleGeneral::rkwardVersionChanged ()) {
+			RK_DEBUG(APP, DL_INFO, "RKWard version changed. Discarding cached package at %s", qPrintable (package));
+			QFile::remove (package);
+		}
+		if (!QFileInfo (package).exists()) {
+			RK_DEBUG(APP, DL_INFO, "Copying rkward R source package to %s", qPrintable (package));
+			RK_ASSERT(QFile::copy (RKCommonFunctions::getRKWardDataDir () + "/rpackages/" + packages[i], package));
+		}
+	}
+
 	RKGlobals::rinter = new RInterface ();
 	new RObjectList ();
 
@@ -604,6 +627,7 @@ void RKWardMainWindow::partChanged (KParts::Part *part) {
 	RK_TRACE (APP);
 
 	if (gui_rebuild_locked) return;
+	if (!part) return;
 	createGUI (part);
 
 	if (!guiFactory ()) {
@@ -762,7 +786,7 @@ bool RKWardMainWindow::doQueryQuit () {
 				return false;
 			}
 		}
-//		lockGUIRebuild (false);  // No need to update GUI anymore (and doing so is potentially asking for trouble, anyway)
+		gui_rebuild_locked = false; // like lockGUIRebuild (false), but does not trigger an immediate rebuild, as we are about to leave, anyway.
 	}
 
 	return true;
@@ -943,3 +967,4 @@ void RKWardMainWindow::setCaption (const QString &) {
 	KParts::MainWindow::setCaption (wcaption);
 }
 
+
