@@ -2,7 +2,7 @@
                           rksettingsmodule  -  description
                              -------------------
     begin                : Wed Jul 28 2004
-    copyright            : (C) 2004, 2007 by Thomas Friedrichsmeier
+    copyright            : (C) 2004-2018 by Thomas Friedrichsmeier
     email                : thomas.friedrichsmeier@kdemail.net
  ***************************************************************************/
 
@@ -32,9 +32,8 @@ Base class for settings modules. Provides some pure virtual calls.
 */
 class RKSettingsModule : public QWidget {
 public:
-    RKSettingsModule (RKSettings *gui, QWidget *parent);
-
-    virtual ~RKSettingsModule ();
+	RKSettingsModule (RKSettings *gui, QWidget *parent);
+	virtual ~RKSettingsModule ();
 
 	bool hasChanges () { return changed; };
 	virtual void applyChanges () = 0;
@@ -56,4 +55,23 @@ friend class RKSettings;
 	static RCommandChain *chain;
 };
 
+#include <functional>
+/** Simple helper class to formalize the API of widgets used for the interactive validation of settings.
+ *  (For quering about settings that may need adjusting on startup. Possibly to be expanded to a "first-run-wizard", in the future). */
+class RKSettingsWizardPage : public QWidget {
+public:
+	RKSettingsWizardPage (QWidget* parent=0) : QWidget (parent) {
+		apply_callback = 0;
+	}
+	~RKSettingsWizardPage () {};
+	void apply () {
+		if (apply_callback) apply_callback();
+	}
+	void setApplyCallback (std::function<void()> callback) {
+		apply_callback = callback;
+	};
+private:
+	std::function<void()> apply_callback;
+};
+
 #endif
