@@ -2,7 +2,7 @@
                           rksettingsmodulecommandeditor  -  description
                              -------------------
     begin                : Tue Oct 23 2007
-    copyright            : (C) 2007, 2010, 2011 by Thomas Friedrichsmeier
+    copyright            : (C) 2007-2019 by Thomas Friedrichsmeier
     email                : thomas.friedrichsmeier@kdemail.net
  ***************************************************************************/
 
@@ -23,6 +23,8 @@ class RKSpinBox;
 class QCheckBox;
 class QLineEdit;
 class QGroupBox;
+class QComboBox;
+class QGridLayout;
 
 /**
 configuration for the Command Editor windows
@@ -33,22 +35,35 @@ class RKSettingsModuleCommandEditor : public RKSettingsModule {
 	Q_OBJECT
 public:
 	RKSettingsModuleCommandEditor (RKSettings *gui, QWidget *parent);
-
 	~RKSettingsModuleCommandEditor ();
 	
 	void applyChanges () override;
 	void save (KConfig *config) override;
-	
+
 	static void saveSettings (KConfig *config);
 	static void loadSettings (KConfig *config);
-	
+	static void validateSettingsInteractive (QList<RKSettingsWizardPage*>*) {};
+
 	QString caption () override;
 
+	enum CompletionCategories {
+		Calltip = 0,
+		Arghint,
+		Object,
+		Filename,
+		AutoWord,
+		N_COMPLETION_CATEGORIES
+	};
+
 /// min number of character to try code completion
-	static int completionMinChars () { return completion_min_chars; };
-	static int completionTimeout () { return completion_timeout; };
-	static bool completionEnabled () { return completion_enabled; };
-	static bool argHintingEnabled () { return arghinting_enabled; };
+	static int autoCompletionMinChars () { return auto_completion_min_chars; };
+	static int autoCompletionTimeout () { return auto_completion_timeout; };
+	static bool autoCompletionEnabled () { return auto_completion_enabled; };
+	static bool autoCompletionCursorActivated () { return (auto_completion_enabled && auto_completion_cursor_activated); };
+	static bool argHintingEnabled () { return isCompletionEnabled (Arghint); };  // TODO: remove me
+	static int completionOptions () { return completion_options; };
+	static bool isCompletionEnabled (CompletionCategories cat) { return completion_type_enabled[cat]; };
+	static bool cursorNavigatesCompletions () { return cursor_navigates_completions; };
 
 	static bool autosaveEnabled () { return autosave_enabled; };
 	static bool autosaveKeep () { return autosave_keep; };
@@ -61,15 +76,27 @@ public:
 public slots:
 	void settingChanged ();
 private:
-	static int completion_min_chars;
-	static int completion_timeout;
-	static bool completion_enabled;
-	static bool arghinting_enabled;
+	void makeCompletionTypeBoxes (const QStringList& label, QGridLayout* layout);
 
-	RKSpinBox* completion_min_chars_box;
-	RKSpinBox* completion_timeout_box;
-	QCheckBox* completion_enabled_box;
-	QCheckBox* arghinting_enabled_box;
+	static int auto_completion_min_chars;
+	static int auto_completion_timeout;
+	static bool auto_completion_enabled;
+	static bool auto_completion_cursor_activated;
+	static bool completion_type_enabled[N_COMPLETION_CATEGORIES];
+	static bool cursor_navigates_completions;
+
+	RKSpinBox* auto_completion_min_chars_box;
+	RKSpinBox* auto_completion_timeout_box;
+	QGroupBox* auto_completion_enabled_box;
+	QCheckBox* auto_completion_cursor_activated_box;
+	QCheckBox* completion_type_enabled_box[N_COMPLETION_CATEGORIES];
+	QCheckBox* cursor_navigates_completions_box;
+
+	static int completion_options;
+
+	QComboBox* completion_list_member_operator_box;
+	QComboBox* completion_slot_operator_box;
+	QComboBox* completion_object_qualification_box;
 
 	static bool autosave_enabled;
 	static bool autosave_keep;
