@@ -2,7 +2,7 @@
                           rksettingsmoduler  -  description
                              -------------------
     begin                : Wed Jul 28 2004
-    copyright            : (C) 2004-2015 by Thomas Friedrichsmeier
+    copyright            : (C) 2004-2018 by Thomas Friedrichsmeier
     email                : thomas.friedrichsmeier@kdemail.net
  ***************************************************************************/
 
@@ -19,6 +19,7 @@
 #include <KLocalizedString>
 #include <KSharedConfig>
 #include <KConfigGroup>
+#include <KUrlRequester>
 
 #include <qlabel.h>
 #include <qcheckbox.h>
@@ -28,7 +29,7 @@
 #include <QGridLayout>
 #include <QPushButton>
 #include <QTextEdit>
-#include <QFileDialog>
+#include <QDialogButtonBox>
 #include <QSpinBox>
 #include <QInputDialog>
 
@@ -56,6 +57,7 @@ bool RKSettingsModuleR::options_checkbounds;
 QString RKSettingsModuleR::options_editor;
 QString RKSettingsModuleR::options_pager;
 QString RKSettingsModuleR::options_further;
+QStringList RKSettingsModuleR::options_addpaths;
 // static constants
 QString RKSettingsModuleR::builtin_editor = "<rkward>";
 // session constants
@@ -206,6 +208,12 @@ RKSettingsModuleR::RKSettingsModuleR (RKSettings *gui, QWidget *parent) : RKSett
 	grid->addWidget (further_input, ++row, 0, 1, 2);
 
 	main_vbox->addStretch ();
+
+	addpaths_selector = new MultiStringSelector (i18n ("Addition search paths for utilities used by R"), this);
+	addpaths_selector->setValues (options_addpaths);
+	connect (addpaths_selector, &MultiStringSelector::listChanged, this, &RKSettingsModuleR::settingChanged);
+	connect (addpaths_selector, &MultiStringSelector::getNewStrings, this, &RKSettingsModuleR::addPaths);
+	main_vbox->addWidget (addpaths_selector);
 }
 
 RKSettingsModuleR::~RKSettingsModuleR() {
@@ -238,6 +246,13 @@ void RKSettingsModuleR::applyChanges () {
 	options_editor = editor_input->currentText ();
 	options_pager = pager_input->currentText ();
 	options_further = further_input->toPlainText ();
+	// normalize system paths before adding
+	QStringList paths = addpaths_selector->getValues ();
+	options_addpaths.clear ();
+	for (int i = 0; i < paths.count (); ++i) {
+		QString path = QDir::cleanPath (paths[i]);
+		if (!options_addpaths.contains (path)) options_addpaths.append (path);
+	}
 
 // apply run time options in R
 	QStringList commands = makeRRunTimeOptionCommands ();
@@ -246,6 +261,31 @@ void RKSettingsModuleR::applyChanges () {
 	}
 }
 
+void RKSettingsModuleR::addPaths(QStringList* string_list) {
+	RK_TRACE (SETTINGS);
+
+	QDialog dialog (this);
+	dialog.setWindowTitle (i18n ("Add System Path Directory"));
+	QVBoxLayout *layout = new QVBoxLayout (&dialog);
+	QLabel *label = new QLabel (i18n ("Specify or select directory to add to the system file path of the running R session"));
+	label->setWordWrap (true);
+	layout->addWidget (label);
+
+	KUrlRequester *req = new KUrlRequester ();
+	req->setMode (KFile::Directory);
+	layout->addWidget (req);
+
+	QDialogButtonBox *buttons = new QDialogButtonBox (QDialogButtonBox::Ok | QDialogButtonBox::Cancel);
+	buttons->button(QDialogButtonBox::Ok)->setText (i18nc ("Add directory to list", "Add"));
+	connect (buttons, &QDialogButtonBox::accepted, &dialog, &QDialog::accept);
+	connect (buttons, &QDialogButtonBox::rejected, &dialog, &QDialog::reject);
+	layout->addWidget (buttons);
+
+	if (dialog.exec () == QDialog::Accepted) {
+		if (!req->text ().isEmpty ()) (*string_list).append (req->text ());
+	}
+}
+
 //static
 QStringList RKSettingsModuleR::makeRRunTimeOptionCommands () {
 	RK_TRACE (SETTINGS);
@@ -270,6 +310,13 @@ QStringList RKSettingsModuleR::makeRRunTimeOptionCommands () {
 	if (options_pager == builtin_editor) list.append ("options (pager=rk.show.files)\n");
 	else list.append ("options (pager=\"" + options_pager + "\")\n");
 	if (!options_further.isEmpty ()) list.append (options_further + '\n');
+	if (!options_addpaths.isEmpty ()) {
+		QString command = "rk.adjust.system.path (add=c(";
+		foreach (const QString &p, options_addpaths) {
+			command.append (RObject::rQuote (p));
+		}
+		list.append (command + "))\n");
+	}
 
 #ifdef __GNUC__
 #	warning TODO make the following options configurable
@@ -277,6 +324,7 @@ QStringList RKSettingsModuleR::makeRRunTimeOptionCommands () {
 	list.append ("options (help_type=\"html\")\n");		// for R 2.10.0 and above
 	list.append ("try ({options (htmlhelp=TRUE); options (chmhelp=FALSE)})\n");	// COMPAT: for R 2.9.x and below
 	list.append ("options (browser=rk.show.html)\n");
+	list.append ("options (askYesNo=rk.askYesNo)\n"); // for R 3.5.0 and above
 
 	return list;
 }
@@ -294,7 +342,7 @@ void RKSettingsModuleR::saveSettings (KConfig *config) {
 	cg.writeEntry ("OutDec", options_outdec);
 	cg.writeEntry ("width", options_width);
 	cg.writeEntry ("warn", options_warn);
-	cg.writeEntry ("maxprint", options_maxprint);
+	cg.writeEntry ("max.print", options_maxprint);
 	cg.writeEntry ("warnings.length", options_warningslength);
 	cg.writeEntry ("keep.source", options_keepsource);
 	cg.writeEntry ("keep.source.pkgs", options_keepsourcepkgs);
@@ -304,6 +352,7 @@ void RKSettingsModuleR::saveSettings (KConfig *config) {
 	cg.writeEntry ("editor", options_editor);
 	cg.writeEntry ("pager", options_pager);
 	cg.writeEntry ("further init commands", options_further);
+	cg.writeEntry ("addsyspaths", options_addpaths);
 }
 
 void RKSettingsModuleR::loadSettings (KConfig *config) {
@@ -323,6 +372,7 @@ void RKSettingsModuleR::loadSettings (KConfig *config) {
 	options_editor = cg.readEntry ("editor", builtin_editor);
 	options_pager = cg.readEntry ("pager", builtin_editor);
 	options_further = cg.readEntry ("further init commands", QString ());
+	options_addpaths = cg.readEntry ("addsyspaths", QStringList ());
 }
 
 //#################################################
@@ -332,6 +382,7 @@ void RKSettingsModuleR::loadSettings (KConfig *config) {
 // static members
 QStringList RKSettingsModuleRPackages::liblocs;
 QStringList RKSettingsModuleRPackages::defaultliblocs;
+QString RKSettingsModuleRPackages::r_libs_user;
 bool RKSettingsModuleRPackages::archive_packages;
 bool RKSettingsModuleRPackages::source_packages;
 QStringList RKSettingsModuleRPackages::package_repositories;
@@ -405,6 +456,29 @@ void RKSettingsModuleRPackages::addLibraryLocation (const QString& new_loc, RCom
 	RKGlobals::rInterface ()->issueCommand (".libPaths (unique (c (" + RObject::rQuote (new_loc) + ", .libPaths ())))", RCommand::App | RCommand::Sync, QString (), 0, 0, chain);
 }
 
+QStringList expandLibLocs (const QStringList &in) {
+	QStringList ret;
+	for (int i = 0; i < in.size (); ++i) {
+		QString dummy = in[i];
+		ret.append (dummy.replace (QLatin1String ("%v"), RKSessionVars::RVersion (true)));
+	}
+	return ret;
+}
+
+QString RKSettingsModuleRPackages::userLibraryLocation () {
+	if (!r_libs_user.isEmpty()) return r_libs_user;
+	return QDir (RKSettingsModuleGeneral::filesPath ()).absoluteFilePath ("library/" + RKSessionVars::RVersion (true));
+}
+
+QStringList RKSettingsModuleRPackages::libraryLocations () {
+	return (QStringList (userLibraryLocation ()) + expandLibLocs (liblocs + defaultliblocs));
+}
+
+QStringList RKSettingsModuleRPackages::addUserLibLocTo (const QStringList& liblocs) {
+	if (!liblocs.contains(userLibraryLocation ())) return (QStringList (userLibraryLocation ()) + liblocs);
+	return liblocs;
+}
+
 void RKSettingsModuleRPackages::settingChanged () {
 	RK_TRACE (SETTINGS);
 	change ();
@@ -412,21 +486,29 @@ void RKSettingsModuleRPackages::settingChanged () {
 
 void RKSettingsModuleRPackages::addLibLoc (QStringList *string_list) {
 	RK_TRACE (SETTINGS);
-#ifdef Q_OS_WIN
-	// KF5 TODO: Still needed?
-	// TODO: Hang on Windows when trying to select any dir using (K|Q)FileDialog::getExistingDirectory (). KDE 4.10
-	QFileDialog dummy (this, i18n ("Add R Library Directory"));
-	dummy.setFileMode (QFileDialog::Directory);
-	dummy.setOptions (QFileDialog::ShowDirsOnly);
-	QString new_string;
-	if (dummy.exec ()) {
-		new_string = dummy.selectedFiles ().value (0);
-	}
-#else
-	QString new_string = QFileDialog::getExistingDirectory (this, i18n ("Add R Library Directory"));
-#endif
-	if (!new_string.isEmpty ()) {
-		(*string_list).append (new_string);
+
+	QDialog dialog (this);
+	dialog.setWindowTitle (i18n ("Add R Library Directory"));
+	QVBoxLayout *layout = new QVBoxLayout (&dialog);
+	QLabel *label = new QLabel (i18n ("Specify or select library location to add.\nNote that locations may contain a '%v', which will expand to the first "
+	                                  "two components of the R version number (e.g. to 3.5), automatically. Including this is recommended, because R packages "
+	                                  "compiled for one version of R will often fail to work correctly in a different version of R."));
+	label->setWordWrap (true);
+	layout->addWidget (label);
+
+	KUrlRequester *req = new KUrlRequester ();
+	req->setText (QDir (RKSettingsModuleGeneral::filesPath ()).absoluteFilePath ("library/%v"));
+	req->setMode (KFile::Directory);
+	layout->addWidget (req);
+
+	QDialogButtonBox *buttons = new QDialogButtonBox (QDialogButtonBox::Ok | QDialogButtonBox::Cancel);
+	buttons->button(QDialogButtonBox::Ok)->setText (i18nc ("Add file to list", "Add"));
+	connect (buttons, &QDialogButtonBox::accepted, &dialog, &QDialog::accept);
+	connect (buttons, &QDialogButtonBox::rejected, &dialog, &QDialog::reject);
+	layout->addWidget (buttons);
+
+	if (dialog.exec () == QDialog::Accepted) {
+		if (!req->text ().isEmpty ()) (*string_list).append (req->text ());
 	}
 }
 
@@ -597,3 +679,63 @@ void RKSettingsModuleRPackages::loadSettings (KConfig *config) {
 #endif
 }
 
+#include <QGroupBox>
+#include <QRadioButton>
+
+void RKSettingsModuleRPackages::validateSettingsInteractive (QList<RKSettingsWizardPage*>* pages) {
+	RK_TRACE (SETTINGS);
+
+	if (RKSettingsModuleGeneral::storedConfigVersion () < RKSettingsModuleGeneral::RKWardConfig_0_7_1) {
+		QString legacy_libloc = QDir (RKSettingsModuleGeneral::filesPath ()).absoluteFilePath ("library");
+		if (liblocs.contains (legacy_libloc)) {
+			RKSettingsWizardPage *page = new RKSettingsWizardPage ();
+			page->setWindowTitle (i18n ("Unversioned library location"));
+			QVBoxLayout *layout = new QVBoxLayout (page);
+			QLabel *label = new QLabel (i18n ("The configured library locations (where R packages will be installed on this system) contains the directory '%1', "
+			                                  "which was suggested as a default library location in earlier versions of RKWard. Use of this directory is no longer "
+			                                  "recommended, as it is not accessible R sessions outside of RKWard (unless configured, explicitly). Also due to the lack "
+			                                  "of an R version number in the directory name, it offers no protection against using packages built for an incompatible "
+			                                  "version of R.", legacy_libloc));
+			label->setWordWrap (true);
+			layout->addWidget (label);
+			// D'uh. QRadioButton supports neither wordwrap nor formatting. Why?
+			QGroupBox *group = new QGroupBox (i18n ("What do you want to do?"));
+			QRadioButton *removebutton = new QRadioButton ();
+			QLabel *removelabel = new QLabel (i18n ("<b>Remove</b> this location from the configuration (it will not be deleted on disk). You will have to "
+			                                        "re-install any packages that you want to keep."));
+			removelabel->setWordWrap (true);
+			QRadioButton *renamebutton = new QRadioButton ();
+			QLabel *renamelabel = new QLabel (i18n ("<b>Rename</b> this location to include the version number of the currently running R. Packages will continue "
+			                                        "to work (if they are compatible with this version of R)."));
+			renamelabel->setWordWrap (true);
+			QRadioButton *keepbutton = new QRadioButton ();
+			QLabel *keeplabel = new QLabel (i18n ("<b>Keep</b> this location (do not change anything)."));
+			keeplabel->setWordWrap (true);
+			QGridLayout *group_layout = new QGridLayout (group);
+			group_layout->addWidget (removebutton, 0, 0);
+			group_layout->addWidget (removelabel, 0, 1);
+			group_layout->addWidget (renamebutton, 1, 0);
+			group_layout->addWidget (renamelabel, 1, 1);
+			group_layout->addWidget (keepbutton, 2, 0);
+			group_layout->addWidget (keeplabel, 2, 1);
+			group_layout->setColumnStretch (1, 1);
+			renamebutton->setChecked (true);
+			layout->addWidget (group);
+
+			page->setApplyCallback([keepbutton, renamebutton, legacy_libloc]() {
+				if (keepbutton->isChecked ()) return;
+
+				liblocs.removeAll (legacy_libloc);
+				if (renamebutton->isChecked ()) {
+					QString new_loc = legacy_libloc + '-' + RKSessionVars::RVersion (true);
+					RKGlobals::rInterface ()->issueCommand (QString ("file.rename(%1, %2)\n").arg (RObject::rQuote (legacy_libloc)).arg (RObject::rQuote (new_loc)), RCommand::App);
+					liblocs.prepend (legacy_libloc + QStringLiteral ("-%v"));
+				}
+
+				RKGlobals::rInterface ()->issueCommand (libLocsCommand(), RCommand::App);
+			});
+
+			pages->append (page);
+		}
+	}
+}
