@@ -2,7 +2,7 @@
                           rksettingsmoduler  -  description
                              -------------------
     begin                : Wed Jul 28 2004
-    copyright            : (C) 2004, 2007, 2009, 2010, 2011, 2012 by Thomas Friedrichsmeier
+    copyright            : (C) 2004-2018 by Thomas Friedrichsmeier
     email                : thomas.friedrichsmeier@kdemail.net
  ***************************************************************************/
 
@@ -39,13 +39,14 @@ class RKSettingsModuleR : public RKSettingsModule {
 public:
 	RKSettingsModuleR (RKSettings *gui, QWidget *parent);
 	~RKSettingsModuleR ();
-	
+
 	void applyChanges () override;
 	void save (KConfig *config) override;
-	
+
 	static void saveSettings (KConfig *config);
 	static void loadSettings (KConfig *config);
-	
+	static void validateSettingsInteractive (QList<RKSettingsWizardPage*>*) {};
+
 	QString caption () override;
 	
 /** generate the commands needed to set the R run time options */
@@ -57,6 +58,8 @@ public:
 	static int getDefaultWidth () { return options_width; };
 public slots:
 	void settingChanged ();
+private slots:
+	void addPaths (QStringList *string_list);
 private:
 	QLineEdit *outdec_input;
 	QSpinBox *width_input;
@@ -71,6 +74,7 @@ private:
 	QComboBox *editor_input;
 	QComboBox *pager_input;
 	QTextEdit *further_input;
+	MultiStringSelector *addpaths_selector;
 
 	static QString options_outdec;
 	static int options_width;
@@ -85,6 +89,7 @@ private:
 	static QString options_editor;
 	static QString options_pager;
 	static QString options_further;
+	static QStringList options_addpaths;
 
 // constants
 	static QString builtin_editor;
@@ -106,12 +111,13 @@ class RKSettingsModuleRPackages : public RKSettingsModule, public RCommandReceiv
 public:
 	RKSettingsModuleRPackages (RKSettings *gui, QWidget *parent);
 	~RKSettingsModuleRPackages ();
-	
+
 	void applyChanges () override;
 	void save (KConfig *config) override;
-	
+
 	static void saveSettings (KConfig *config);
 	static void loadSettings (KConfig *config);
+	static void validateSettingsInteractive (QList<RKSettingsWizardPage*>*);
 
 /** generate the commands needed to set the R run time options */
 	static QStringList makeRRunTimeOptionCommands ();
@@ -119,7 +125,10 @@ public:
 	static bool archivePackages () { return archive_packages; }
 	static QString pkgTypeOption ();
 	static void addLibraryLocation (const QString& new_loc, RCommandChain *chain);
-	static QStringList libraryLocations () { return (liblocs + defaultliblocs); };
+	static QStringList libraryLocations ();
+/** Add a reasonable user writable path to the given library locations. */
+	static QStringList addUserLibLocTo (const QStringList &liblocs);
+	static QString userLibraryLocation ();
 
 /** returns the list of packages which are essential to rkward. This is hard-coded. */
 	static QStringList essentialPackages () { return essential_packages.split ("\n"); };
@@ -148,6 +157,7 @@ private:
 	static QStringList package_repositories;
 
 	friend class RInterface;
+	static QString r_libs_user;
 	static QStringList defaultliblocs;
 	static QString essential_packages;
 };
