@@ -2,7 +2,7 @@
                           rkcommandeditorwindow  -  description
                              -------------------
     begin                : Mon Aug 30 2004
-    copyright            : (C) 2004-2017 by Thomas Friedrichsmeier
+    copyright            : (C) 2004-2019 by Thomas Friedrichsmeier
     email                : thomas.friedrichsmeier@kdemail.net
  ***************************************************************************/
 
@@ -24,9 +24,6 @@
 
 #include <ktexteditor/view.h>
 #include <ktexteditor/document.h>
-#include <ktexteditor/codecompletionmodel.h>
-#include <ktexteditor/codecompletioninterface.h>
-#include <ktexteditor/codecompletionmodelcontrollerinterface.h>
 #include <ktexteditor/movingrange.h>
 #include <ktexteditor/movinginterface.h>
 
@@ -39,7 +36,9 @@ class QCloseEvent;
 class QFrame;
 class QLabel;
 class QAction;
-class QAction;
+class QActionGroup;
+class QTemporaryDir;
+class QFile;
 class KActionMenu;
 class RKCommandEditorWindow;
 class KActionCollection;
@@ -102,31 +101,10 @@ private:
 	QLabel *arghints_popup;
 };
 
-class RKCodeCompletionModel : public KTextEditor::CodeCompletionModel, public KTextEditor::CodeCompletionModelControllerInterface {
-	Q_OBJECT
-	Q_INTERFACES(KTextEditor::CodeCompletionModelControllerInterface)
-public:
-	explicit RKCodeCompletionModel (RKCommandEditorWindow* parent);
-	~RKCodeCompletionModel ();
-
-	KTextEditor::Range completionRange (KTextEditor::View *view, const KTextEditor::Cursor &position) override;
-	QString filterString (KTextEditor::View *, const KTextEditor::Range &, const KTextEditor::Cursor &) override { return QString (); };
-
-	void updateCompletionList (const QString& symbol);
-	void completionInvoked (KTextEditor::View *, const KTextEditor::Range &, InvocationType) override;
-	void executeCompletionItem (KTextEditor::View *view, const KTextEditor::Range &word, const QModelIndex &index) const override;
-	QVariant data (const QModelIndex& index, int role=Qt::DisplayRole) const override;
-
-	bool isEmpty () const { return names.isEmpty (); };
-private:
-	QList<QIcon> icons;
-	QStringList names;
-	QString current_symbol;
-	RKCommandEditorWindow *command_editor;
-};
-
-class QTimer;
 class RKJobSequence;
+class RKXMLGUIPreviewArea;
+class RKPreviewManager;
+class RKCompletionManager;
 
 /**
 	\brief Provides an editor window for R-commands, as well as a text-editor window in general.
@@ -172,16 +150,11 @@ public:
 
 	QString provideContext (int line_rev) override;
 	void currentHelpContext (QString* symbol, QString* package) override;
-	QString currentCompletionWord () const;
 
 	void highlightLine (int linenum);
 public slots:
 /** update Tab caption according to the current url. Display the filename-component of the URL, or - if not available - a more elaborate description of the url. Also appends a "[modified]" if appropriate */
 	void updateCaption ();
-/** called whenever it might be appropriate to show a code completion box. The box is not shown immediately, but only after a timeout (if at all) */
-	void tryCompletionProxy (KTextEditor::Document*);
-/** show a code completion box if appropriate. Use tryCompletionProxy () instead, which will call this function after a timeout */
-	void tryCompletion ();
 	void setPopupMenu ();
 	void focusIn (KTextEditor::View *);
 /** run the currently selected command(s) or line */
@@ -216,24 +189,24 @@ private slots:
 /** run a block */
 	void runBlock ();
 	void clearUnusedBlocks ();
+/** handler to control when autosaves should be created, preview should be updated */
+	void textChanged ();
+/** Render the (.Rmd) current script */
+	void doRenderPreview ();
 /** creates an autosave file */
 	void doAutoSave ();
 /** handler to control when autosaves should be created */
 	void autoSaveHandlerModifiedChanged ();
-/** handler to control when autosaves should be created */
-	void autoSaveHandlerTextChanged ();
 /** handle any errors during auto-saving */
 	void autoSaveHandlerJobFinished (RKJobSequence* seq);
+/** document was saved. Update preview, if appropriate */
+	void documentSaved ();
 private:
 	KTextEditor::Cursor saved_scroll_position;
 	KTextEditor::Document *m_doc;
 	KTextEditor::View *m_view;
-	KTextEditor::CodeCompletionInterface *cc_iface;
 	KTextEditor::MovingInterface *smart_iface;
 	RKFunctionArgHinter *hinter;
-	RKCodeCompletionModel *completion_model;
-
-	QTimer *completion_timer;
 
 	void initializeActions (KActionCollection* ac);
 
@@ -258,16 +231,35 @@ private:
 
 	QAction* action_run_all;
 	QAction* action_run_current;
+	QActionGroup* preview_modes;
+	QAction* action_no_preview;
+	QAction* action_preview_as_you_type;
+
+	enum PreviewMode {
+		NoPreview,
+		RMarkdownPreview,
+		RKOutputPreview,
+		GraphPreview,
+		ConsolePreview,
+	};
 
 	QAction* action_setwd_to_script;
 
 	QUrl previous_autosave_url;
-	QTimer* autosave_timer;
+	QTimer autosave_timer;
 
 	QUrl delete_on_close;
 
 	QString _id;
 	static QMap<QString, KTextEditor::Document*> unnamed_documents;
+
+	RKXMLGUIPreviewArea *preview;
+	QTimer preview_timer;
+	RKPreviewManager *preview_manager;
+	QTemporaryDir *preview_dir;
+	QFile *preview_input_file;
+	void changePreviewMode (QAction* mode);
+	void discardPreview ();
 };
 
 /** Simple class to provide HTML highlighting for arbitrary R code. */
