@@ -2,7 +2,7 @@
                           rkfilebrowser  -  description
                              -------------------
     begin                : Thu Apr 26 2007
-    copyright            : (C) 2007-2016 by Thomas Friedrichsmeier
+    copyright            : (C) 2007-2018 by Thomas Friedrichsmeier
     email                : thomas.friedrichsmeier@kdemail.net
  ***************************************************************************/
 
@@ -39,6 +39,8 @@
 #include <QInputDialog>
 
 #include "rkworkplace.h"
+#include "../rkglobals.h"
+#include "../rbackend/rkrinterface.h"
 #include "../rkward.h"
 #include "../misc/rkdummypart.h"
 
@@ -83,10 +85,7 @@ void RKFileBrowser::showEvent (QShowEvent *e) {
 	}
 
 	RKMDIWindow::showEvent (e);
-}
-
-void RKFileBrowser::currentWDChanged () {
-	RK_TRACE (APP);
+	real_widget->syncToWD();
 }
 
 /////////////////// RKFileBrowserWidget ////////////////////
@@ -114,7 +113,7 @@ RKFileBrowserWidget::RKFileBrowserWidget (QWidget *parent) : QWidget (parent) {
 	dir->setPreviewWidget (0);
 	KConfigGroup config = KSharedConfig::openConfig ()->group ("file browser window");
 	dir->readConfig (config);
-	dir->setView (KFile::Default);
+	dir->setView (KFile::Tree);
 	connect (RKWardMainWindow::getMain (), &RKWardMainWindow::aboutToQuitRKWard, this, &RKFileBrowserWidget::saveConfig);
 	layout->addWidget (dir);
 
@@ -122,6 +121,11 @@ RKFileBrowserWidget::RKFileBrowserWidget (QWidget *parent) : QWidget (parent) {
 	toolbar->addAction (dir->actionCollection ()->action ("back"));
 	toolbar->addAction (dir->actionCollection ()->action ("forward"));
 	toolbar->addAction (dir->actionCollection ()->action ("home"));
+	QAction* action = new QAction (QIcon::fromTheme ("folder-sync"), i18n ("Working directory"), this);
+	action->setToolTip (action->text ());
+	connect(action, &QAction::triggered, this, [=] () { follow_working_directory = true; syncToWD(); });
+	toolbar->addAction (action);
+	toolbar->addSeparator ();
 	toolbar->addAction (dir->actionCollection ()->action ("short view"));
 	toolbar->addAction (dir->actionCollection ()->action ("tree view"));
 	toolbar->addAction (dir->actionCollection ()->action ("detailed view"));
@@ -138,6 +142,7 @@ RKFileBrowserWidget::RKFileBrowserWidget (QWidget *parent) : QWidget (parent) {
 	connect (urlbox, &KUrlComboBox::urlActivated, this, &RKFileBrowserWidget::urlChangedInCombo);
 
 	connect (dir, &KDirOperator::fileSelected, this, &RKFileBrowserWidget::fileActivated);
+	connect (RKGlobals::rInterface (), &RInterface::backendWorkdirChanged, this, &RKFileBrowserWidget::syncToWD);
 
 	setURL (QUrl::fromLocalFile (QDir::currentPath ()));
 }
@@ -146,6 +151,13 @@ RKFileBrowserWidget::~RKFileBrowserWidget () {
 	RK_TRACE (APP);
 }
 
+void RKFileBrowserWidget::syncToWD () {
+	RK_TRACE (APP);
+
+	if (!follow_working_directory) return;
+	if (isVisible()) setURL (QUrl::fromLocalFile (QDir::currentPath ()));
+}
+
 void RKFileBrowserWidget::rename () {
 	RK_TRACE (APP);
 
@@ -197,12 +209,14 @@ void RKFileBrowserWidget::setURL (const QUrl &url) {
 
 	urlbox->setUrl (url);
 	dir->setUrl (url, true);
+	follow_working_directory = (url.path() == QDir::currentPath ());
 }
 
 void RKFileBrowserWidget::urlChangedInView (const QUrl &url) {
 	RK_TRACE (APP);
 
 	urlbox->setUrl (url);
+	follow_working_directory = (url.path() == QDir::currentPath ());
 }
 
 void RKFileBrowserWidget::stringChangedInCombo (const QString &url) {
